#
# Models of partisan modularity
#

# required packages
library(ggplot2)
library(arm)
library(texreg)
library(tidyr)

# network descriptors for N = 96 legislatures
r = read.csv("data.csv", stringsAsFactors = FALSE)

# rescaled values
r$bills = r$bills / 1000
r$nodes = r$nodes / 100
r$size  = r$size  / 100

# remove Austria (disassortative mixing)
m = r[ r$cty != "AT", ]

# plot theme
theme_models = theme_bw(18) +
  theme(strip.background = element_rect(color = "grey90", fill = "grey90"),
        legend.position = "bottom",
        legend.title = element_text(size = rel(1)),
        legend.text = element_text(size = rel(1)),
        panel.grid = element_blank())

#
# Distribution of partisan modularity
#

summary(m$mod_raw)
summary(m$mod_gsw)
 
qplot(data = gather(m[, c("mod_raw", "mod_gsw") ]),
      x = value, lty = key, geom = "density") +
  scale_linetype_discrete("", labels = c("Modularité partisane non pondérée", 
                                         "Modularité partisane pondérée")) +
  labs(y = "Density\n", x = "\nPartisan modularity") +
  theme_models +
  theme(legend.direction = "vertical")

ggsave("densité.png", width = 9, height = 9)
ggsave("densité.pdf", width = 9, height = 9)

#
# Models M1-M4: DV = unweighted partisan modularity (raw cosponsorship counts)
#

# Model 1 -- all countries

M1 = lm(mod_raw ~ n_parties + bills + nodes + density +
           mu_lr + sd_lr + p_females + p_freshmen + p_indepdt + 
           duration,
         data = r)

# Model 2 -- excl. Austria

M2 = lm(mod_raw ~ n_parties + bills + nodes + density +
          mu_lr + sd_lr + p_females + p_freshmen + p_indepdt + 
          duration,
        data = m)

# Model 3 -- adding committee information, excl. Belgium

M3 = lm(mod_raw ~ n_parties + n_comm + m_comm + bills + nodes + density +
          mu_lr + sd_lr + p_females + p_freshmen + p_indepdt + 
          duration,
        data = r)

# Model 4 -- adding committee information, excl. Belgium, excl. Austria

M4 = lm(mod_raw ~ n_parties + n_comm + m_comm + bills + nodes + density +
          mu_lr + sd_lr + p_females + p_freshmen + p_indepdt + 
          duration,
        data = m)

print(screenreg(list(M1, M2, M3, M4)))

# RMSE
summary(M1)$sigma
summary(M2)$sigma
summary(M3)$sigma
summary(M4)$sigma

#
# Residuals v. fitted values in Model M4
#

qplot(y = resid(M4), x = fitted(M4)) + 
  # geom_smooth(se = FALSE, color = "black") +
  geom_hline(y = mean(resid(M4)), color = "black") +
  geom_hline(y = mean(resid(M4)) + sd(resid(M4)), color = "black", lty = "dashed") +
  geom_hline(y = mean(resid(M4)) - sd(resid(M4)), color = "black", lty = "dashed") +
  labs(y = "Résidus\n", x = "\nValeurs prédites") +
  theme_models

ggsave("m4_rvf.png", width = 9, height = 9)
ggsave("m4_rvf.pdf", width = 9, height = 9)

#
# Marginal effect of party positions in Model 4
#

M4pred = seq(min(m$sd_lr), max(m$sd_lr), length.out = 1000)
M4pred = data.frame(x = M4pred, yhat = coef(M4)["sd_lr"] * M4pred)

cov = vcov(M4)
se = sqrt(cov["sd_lr", "sd_lr"] + 
            M4pred$x^2 * cov["sd_lr", "sd_lr"] + 
            2 * M4pred$x * cov["sd_lr", "sd_lr"])

# 90% CI
M4pred$ub = M4pred$yhat + 1.64 * se
M4pred$lb = M4pred$yhat - 1.64 * se

qplot(data = M4pred, y = yhat, x = x, geom = "line") + 
  geom_rug(data = M4, aes(x = sd_lr, y = NULL)) +
  geom_line(aes(y = ub), lty = "dashed") +
  geom_line(aes(y = lb), lty = "dashed") +
  labs(y = "Effet marginal sur la modularité partisane\n",
       x = "\nÉcart-type des positions partisanes gauche-droite") +
  theme_models

ggsave("m4_mfx.png", width = 9, height = 9)
ggsave("m4_mfx.pdf", width = 9, height = 9)

#
# Predicted partisan modularity in Model M4
#

x = seq(min(m$sd_lr), max(m$sd_lr), length.out = round(250 * diff(range(m$sd_lr))))
sim = sim(M4, 1000)

yhat =
  coef(M4)[1] + 
  coef(M4)[2] * mean(m$n_parties, na.rm = TRUE) +
  coef(M4)[3] * mean(m$n_comm, na.rm = TRUE) + 
  coef(M4)[4] * mean(m$m_comm, na.rm = TRUE) + 
  coef(M4)[5] * mean(m$bills, na.rm = TRUE) + 
  coef(M4)[6] * mean(m$nodes, na.rm = TRUE) + 
  coef(M4)[7] * mean(m$density, na.rm = TRUE) + 
  coef(M4)[8] * mean(m$mu_lr, na.rm = TRUE) + 
  coef(M4)[9] * x + 
  coef(M4)[10] * mean(m$p_females, na.rm = TRUE) + 
  coef(M4)[11] * mean(m$p_freshmen, na.rm = TRUE) + 
  coef(M4)[12] * mean(m$p_indepdt, na.rm = TRUE) + 
  coef(M4)[13] * mean(m$duration, na.rm = TRUE)

hi = numeric(length(x))
lo = numeric(length(x))

cat("\nSimulating M4: ")
for(j in length(x):1) {
  
  pred2 = numeric(1000)
  for(i in 1:length(pred2)) {
    
    pred2[i] =
      coef(sim)[i, 1] + 
      coef(sim)[i, 2] * mean(m$n_parties, na.rm = TRUE) +
      coef(sim)[i, 3] * mean(m$n_comm, na.rm = TRUE) + 
      coef(sim)[i, 4] * mean(m$m_comm, na.rm = TRUE) + 
      coef(sim)[i, 5] * mean(m$bills, na.rm = TRUE) + 
      coef(sim)[i, 6] * mean(m$nodes, na.rm = TRUE) + 
      coef(sim)[i, 7] * mean(m$density, na.rm = TRUE) + 
      coef(sim)[i, 8] * mean(m$mu_lr, na.rm = TRUE) + 
      coef(sim)[i, 9] * x[ j ] + 
      coef(sim)[i, 10] * mean(m$p_females, na.rm = TRUE) + 
      coef(sim)[i, 11] * mean(m$p_freshmen, na.rm = TRUE) + 
      coef(sim)[i, 12] * mean(m$p_indepdt, na.rm = TRUE) + 
      coef(sim)[i, 13] * mean(m$duration, na.rm = TRUE)
    
  }
  
  lo[ j ] = quantile(pred2, 0.025)
  hi[ j ] = quantile(pred2, 0.975)
  
  if(!j %% 10)
    cat(j, "... ")
  
}

qplot(y = yhat, x = x, geom = "blank") +
  geom_ribbon(aes(ymin = lo, ymax = hi), fill = "grey90") +
  geom_line() +
  geom_line(aes(y = hi), lty = "dashed") +
  geom_line(aes(y = lo), lty = "dashed") +
  geom_rug(data = m, aes(y = NULL, x = sd_lr)) +
  labs(y = "Modularité partisane prédite\n",
       x = "\nÉcart-type des positions partisanes gauche-droite") +
  theme_models

ggsave("m4_pred.png", width = 9, height = 9)
ggsave("m4_pred.pdf", width = 9, height = 9)

M4pred = data.frame(model = "Modularité partisane non pondérée", x = x, yhat, hi, lo)

#
# Models W1-W4: DV = weighted partisan modularity (weighted propensity to cosponsor)
#

# Model 1 -- all countries

W1 = lm(mod_gsw ~ n_parties + bills + nodes + density +
          mu_lr + sd_lr + p_females + p_freshmen + p_indepdt + 
          duration,
        data = r)

# Model 2 -- excl. Austria

W2 = lm(mod_gsw ~ n_parties + bills + nodes + density +
          mu_lr + sd_lr + p_females + p_freshmen + p_indepdt + 
          duration,
        data = m)

# Model 3 -- adding committee information, excl. Belgium

W3 = lm(mod_gsw ~ n_parties + n_comm + m_comm + bills + nodes + density +
          mu_lr + sd_lr + p_females + p_freshmen + p_indepdt + 
          duration,
        data = r)

# Model 4 -- adding committee information, excl. Belgium, excl. Austria

W4 = lm(mod_gsw ~ n_parties + n_comm + m_comm + bills + nodes + density +
          mu_lr + sd_lr + p_females + p_freshmen + p_indepdt + 
          duration,
        data = m)

print(screenreg(list(W1, W2, W3, W4)))

# RMSE
summary(W1)$sigma
summary(W2)$sigma
summary(W3)$sigma
summary(W4)$sigma

#
# Residuals v. fitted values in Model W4
#

qplot(y = resid(W4), x = fitted(W4)) + 
  # geom_smooth(se = FALSE, color = "black") +
  geom_hline(y = mean(resid(W4)), color = "black") +
  geom_hline(y = mean(resid(W4)) + sd(resid(W4)), color = "black", lty = "dashed") +
  geom_hline(y = mean(resid(W4)) - sd(resid(W4)), color = "black", lty = "dashed") +
  labs(y = "Résidus\n", x = "\nValeurs prédites") +
  theme_linedraw(18) +
  theme(panel.grid = element_blank())

ggsave("w4_rvf.png", width = 9, height = 9)
ggsave("w4_rvf.pdf", width = 9, height = 9)

W4pred = seq(min(m$sd_lr), max(m$sd_lr), length.out = 1000)
W4pred = data.frame(x = W4pred, yhat = coef(W4)["sd_lr"] * W4pred)

cov = vcov(W4)
se = sqrt(cov["sd_lr", "sd_lr"] + 
            W4pred$x^2 * cov["sd_lr", "sd_lr"] + 
            2 * W4pred$x * cov["sd_lr", "sd_lr"])

# 90% CI
W4pred$ub = W4pred$yhat + 1.64 * se
W4pred$lb = W4pred$yhat - 1.64 * se

qplot(data = W4pred, y = yhat, x = x, geom = "line") + 
  geom_rug(data = W4, aes(x = sd_lr, y = NULL)) +
  geom_line(aes(y = ub), lty = "dashed") +
  geom_line(aes(y = lb), lty = "dashed") +
  labs(y = "Effet marginal sur la modularité partisane\n",
       x = "\nÉcart-type des positions partisanes gauche-droite") +
  theme_models

ggsave("w4_mfx.png", width = 9, height = 9)
ggsave("w4_mfx.pdf", width = 9, height = 9)

#
# Predicted partisan modularity in Model W4
#

cat("\nSimulating W4: ")
x = seq(min(m$sd_lr), max(m$sd_lr), length.out = round(250 * diff(range(m$sd_lr))))
sim = sim(W4, 1000)

yhat =
  coef(W4)[1] + 
  coef(W4)[2] * mean(m$n_parties, na.rm = TRUE) +
  coef(W4)[3] * mean(m$n_comm, na.rm = TRUE) + 
  coef(W4)[4] * mean(m$m_comm, na.rm = TRUE) + 
  coef(W4)[5] * mean(m$bills, na.rm = TRUE) + 
  coef(W4)[6] * mean(m$nodes, na.rm = TRUE) + 
  coef(W4)[7] * mean(m$density, na.rm = TRUE) + 
  coef(W4)[8] * mean(m$mu_lr, na.rm = TRUE) + 
  coef(W4)[9] * x + 
  coef(W4)[10] * mean(m$p_females, na.rm = TRUE) + 
  coef(W4)[11] * mean(m$p_freshmen, na.rm = TRUE) + 
  coef(W4)[12] * mean(m$p_indepdt, na.rm = TRUE) + 
  coef(W4)[13] * mean(m$duration, na.rm = TRUE)

hi = numeric(length(x))
lo = numeric(length(x))

for(j in length(x):1) {
  
  pred2 = numeric(1000)
  for(i in 1:length(pred2)) {
    
    pred2[i] =
      coef(sim)[i, 1] + 
      coef(sim)[i, 2] * mean(m$n_parties, na.rm = TRUE) +
      coef(sim)[i, 3] * mean(m$n_comm, na.rm = TRUE) + 
      coef(sim)[i, 4] * mean(m$m_comm, na.rm = TRUE) + 
      coef(sim)[i, 5] * mean(m$bills, na.rm = TRUE) + 
      coef(sim)[i, 6] * mean(m$nodes, na.rm = TRUE) + 
      coef(sim)[i, 7] * mean(m$density, na.rm = TRUE) + 
      coef(sim)[i, 8] * mean(m$mu_lr, na.rm = TRUE) + 
      coef(sim)[i, 9] * x[ j ] + 
      coef(sim)[i, 10] * mean(m$p_females, na.rm = TRUE) + 
      coef(sim)[i, 11] * mean(m$p_freshmen, na.rm = TRUE) + 
      coef(sim)[i, 12] * mean(m$p_indepdt, na.rm = TRUE) + 
      coef(sim)[i, 13] * mean(m$duration, na.rm = TRUE)
    
  }
  
  lo[ j ] = quantile(pred2, 0.025)
  hi[ j ] = quantile(pred2, 0.975)
  
  if(!j %% 10)
    cat(j, "... ")
  
}

qplot(y = yhat, x = x, geom = "blank") +
  geom_ribbon(aes(ymin = lo, ymax = hi), fill = "grey90") +
  geom_line() +
  geom_line(aes(y = hi), lty = "dashed") +
  geom_line(aes(y = lo), lty = "dashed") +
  geom_rug(data = m, aes(y = NULL, x = sd_lr)) +
  labs(y = "Modularité partisane prédite\n",
       x = "\nÉcart-type des positions partisanes gauche-droite") +
  theme_linedraw(18) +
  theme(panel.grid = element_blank())

ggsave("w4_pred.png", width = 9, height = 9)
ggsave("w4_pred.pdf", width = 9, height = 9)

W4pred = data.frame(model = "Modularité partisane pondérée", x = x, yhat, hi, lo)

#
# Combined prediction plots
#

qplot(data = rbind(M4pred, W4pred), y = yhat, x = x, geom = "blank") +
  geom_ribbon(aes(ymin = lo, ymax = hi), fill = "grey90") +
  geom_line() +
  geom_line(aes(y = hi), lty = "dashed") +
  geom_line(aes(y = lo), lty = "dashed") +
  geom_rug(data = m, aes(y = NULL, x = sd_lr)) +
  labs(y = "Modularité partisane prédite\n",
       x = "\nÉcart-type des positions partisanes gauche-droite") +
  facet_wrap(~ model) +
  theme_models

ggsave("modèles.png", width = 18, height = 9)
ggsave("modèles.pdf", width = 18, height = 9)

#
# Have a nice day
#
